@@ -1,5 +1,5 @@
 /*
- * $Id: config.c 4591 2010-03-27 06:28:12Z tatebe $
+ * $Id: config.c 4769 2010-06-23 19:45:14Z tatebe $
  */
 
 #include <sys/types.h>
@@ -351,6 +351,8 @@ char *gfarm_localfs_datadir = NULL;
 
 /* miscellaneous */
 #define GFARM_LOG_MESSAGE_VERBOSE_DEFAULT	0
+#define GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT 30 /* 30 seconds */
+#define GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT 30 /* 30 seconds */
 #define GFARM_ATTR_CACHE_LIMIT_DEFAULT		40000 /* 40,000 entries */
 #define GFARM_ATTR_CACHE_TIMEOUT_DEFAULT	1000 /* 1,000 milli second */
 #define GFARM_SCHEDULE_CACHE_TIMEOUT_DEFAULT 600 /* 10 minutes */
@@ -365,6 +367,8 @@ char *gfarm_localfs_datadir = NULL;
 #define MISC_DEFAULT -1
 int gfarm_log_level = MISC_DEFAULT;
 int gfarm_log_message_verbose = MISC_DEFAULT;
+int gfarm_no_file_system_node_timeout = MISC_DEFAULT;
+int gfarm_gfmd_reconnection_timeout = MISC_DEFAULT;
 int gfarm_attr_cache_limit = MISC_DEFAULT;
 int gfarm_attr_cache_timeout = MISC_DEFAULT;
 int gfarm_schedule_cache_timeout = MISC_DEFAULT;
@@ -1216,6 +1220,10 @@ parse_one_line(char *s, char *p, char **op)
 	} else if (strcmp(s, o = "log_message_verbose_level") == 0) {
 		e = parse_set_misc_int(p, &gfarm_log_message_verbose);
 		gflog_set_message_verbose(gfarm_log_message_verbose);
+	} else if (strcmp(s, o = "no_file_system_node_timeout") == 0) {
+		e = parse_set_misc_int(p, &gfarm_no_file_system_node_timeout);
+	} else if (strcmp(s, o = "gfmd_reconnection_timeout") == 0) {
+		e = parse_set_misc_int(p, &gfarm_gfmd_reconnection_timeout);
 	} else if (strcmp(s, o = "attr_cache_limit") == 0) {
 		e = parse_set_misc_int(p, &gfarm_attr_cache_limit);
 	} else if (strcmp(s, o = "attr_cache_timeout") == 0) {
@@ -1347,6 +1355,12 @@ gfarm_config_set_default_misc(void)
 		gfarm_log_message_verbose = GFARM_LOG_MESSAGE_VERBOSE_DEFAULT;
 	gflog_set_message_verbose(gfarm_log_message_verbose);
 
+	if (gfarm_no_file_system_node_timeout == MISC_DEFAULT)
+		gfarm_no_file_system_node_timeout =
+		    GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT;
+	if (gfarm_gfmd_reconnection_timeout == MISC_DEFAULT)
+		gfarm_gfmd_reconnection_timeout =
+		    GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT;
 	if (gfarm_attr_cache_limit == MISC_DEFAULT)
 		gfarm_attr_cache_limit = GFARM_ATTR_CACHE_LIMIT_DEFAULT;
 	if (gfarm_attr_cache_timeout == MISC_DEFAULT)
