@@ -11,7 +11,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#include <time.h>
+#include <sys/time.h>
 
 #include <gfarm/gfarm_config.h>
 #include <gfarm/gflog.h>
@@ -250,6 +250,7 @@ gfm_client_connection_acquire(const char *hostname, int port,
 	int created;
 	unsigned int sleep_interval = 1;	/* 1 sec */
 	unsigned int sleep_max_interval = 512;	/* about 8.5 min */
+	struct timeval expiration_time;
 
 	e = gfp_cached_connection_acquire(&gfm_server_cache,
 	    hostname, port, &cache_entry, &created);
@@ -261,7 +262,10 @@ gfm_client_connection_acquire(const char *hostname, int port,
 	}
 	e = gfm_client_connection0(hostname, port, cache_entry, gfm_serverp,
 	    NULL);
-	while (IS_CONNECTION_ERROR(e)) {
+	gettimeofday(&expiration_time, NULL);
+	expiration_time.tv_sec += gfarm_gfmd_reconnection_timeout;
+	while (IS_CONNECTION_ERROR(e) &&
+	       !gfarm_timeval_is_expired(&expiration_time)) {
 		gflog_warning(GFARM_MSG_1000058,
 		    "connecting to gfmd at %s:%d failed, "
 		    "sleep %d sec: %s", hostname, port, sleep_interval,
@@ -271,8 +275,6 @@ gfm_client_connection_acquire(const char *hostname, int port,
 			gfm_serverp, NULL);
 		if (sleep_interval < sleep_max_interval)
 			sleep_interval *= 2;
-		else
-			break; /* give up */
 	}
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_error(GFARM_MSG_1000059,
