@@ -1,5 +1,5 @@
 /*
- * $Id: gfs_pio.c 4310 2010-01-12 10:14:59Z ookuma $
+ * $Id: gfs_pio.c 4795 2010-06-27 10:08:43Z tatebe $
  */
 
 #include <sys/types.h> /* mode_t */
@@ -341,7 +341,10 @@ gfs_pio_fillbuf(GFS_File gf, size_t size)
 	} else
 		gfs_pio_purge(gf);
 
-	assert(gf->io_offset == gf->offset);
+	if (gf->io_offset != gf->offset) {
+		gf->mode &= ~GFS_FILE_MODE_CALC_DIGEST;
+		gf->io_offset = gf->offset;
+	}
 
 	e = (*gf->ops->view_pread)(gf, gf->buffer, size, gf->io_offset, &len);
 	if (e != GFARM_ERR_NO_ERROR) {
@@ -368,7 +371,6 @@ do_write(GFS_File gf, const char *buffer, size_t length,
 		return (GFARM_ERR_NO_ERROR);
 	}
 	if (gf->io_offset != gf->offset) {
-		/* this happens when switching from reading to writing */
 		gf->mode &= ~GFS_FILE_MODE_CALC_DIGEST;
 		gf->io_offset = gf->offset;
 	}
@@ -434,7 +436,7 @@ gfs_pio_seek(GFS_File gf, gfarm_off_t offset, int whence, gfarm_off_t *resultp)
 		where = offset + gf->offset + gf->p;
 		break;
 	case GFARM_SEEK_END:
-		/* XXX FIXME: ask the file size to gfmd. */
+		/* XXX FIXME: ask the file size to gfsd. */
 		e = gf->error = GFARM_ERR_FUNCTION_NOT_IMPLEMENTED;
 		goto finish;
 	default:
@@ -1107,6 +1109,8 @@ gfs_pio_stat(GFS_File gf, struct gfs_stat *st)
 		return (e);
 
 	if (gfs_pio_is_view_set(gf)) {
+		if ((gf->mode & GFS_FILE_MODE_WRITE) != 0)
+			gfs_pio_flush(gf);
 		e = (*gf->ops->view_fstat)(gf, st);
 		if (e != GFARM_ERR_NO_ERROR)
 			gf->error = e;
