@@ -1,7 +1,7 @@
 /*
  * pio operations for file fragments or programs
  *
- * $Id: gfs_pio_section.c 4328 2010-01-21 14:31:23Z tatebe $
+ * $Id: gfs_pio_section.c 4801 2010-06-28 06:04:40Z tatebe $
  */
 
 #include <assert.h>
@@ -25,6 +25,7 @@
 #include "host.h"
 #include "config.h"
 #include "gfm_client.h"
+#include "gfm_schedule.h"
 #include "gfs_client.h"
 #include "gfs_proto.h"
 #include "gfs_io.h"
@@ -300,64 +301,6 @@ finish:
 
 #endif /* not yet in gfarm v2 */
 
-struct gfm_schedule_file_closure {
-	int nhosts;
-	struct gfarm_host_sched_info *infos;
-};
-
-static gfarm_error_t
-gfm_schedule_file_request(struct gfm_connection *gfm_server, void *closure)
-{
-	gfarm_error_t e = gfm_client_schedule_file_request(gfm_server, "");
-
-	if (e != GFARM_ERR_NO_ERROR)
-		gflog_warning(GFARM_MSG_1000106, "schedule_file request: %s",
-		    gfarm_error_string(e));
-	return (e);
-}
-
-static gfarm_error_t
-gfm_schedule_file_result(struct gfm_connection *gfm_server, void *closure)
-{
-	struct gfm_schedule_file_closure *c = closure;
-	gfarm_error_t e = gfm_client_schedule_file_result(gfm_server,
-	    &c->nhosts, &c->infos);
-
-#if 1 /* DEBUG */
-	if (e != GFARM_ERR_NO_ERROR)
-		gflog_debug(GFARM_MSG_1000107, "schedule_file result: %s",
-		    gfarm_error_string(e));
-#endif
-	return (e);
-}
-
-static void
-gfm_schedule_file_cleanup(struct gfm_connection *gfm_server, void *closure)
-{
-	struct gfm_schedule_file_closure *c = closure;
-
-	gfarm_host_sched_info_free(c->nhosts, c->infos);
-}
-
-static gfarm_error_t
-gfm_schedule_file(struct gfm_connection *gfm_server, gfarm_int32_t fd,
-	int *nhostsp, struct gfarm_host_sched_info **infosp)
-{
-	gfarm_error_t e;
-	struct gfm_schedule_file_closure closure;
-
-	e = gfm_client_compound_fd_op(gfm_server, fd,
-	    gfm_schedule_file_request,
-	    gfm_schedule_file_result,
-	    gfm_schedule_file_cleanup,
-	    &closure);
-	if (e == GFARM_ERR_NO_ERROR) {
-		*nhostsp = closure.nhosts;
-		*infosp = closure.infos;
-	}
-	return (e);
-}
-
 static gfarm_error_t
 connect_and_open(GFS_File gf, const char *hostname, int port)
 {
@@ -536,9 +479,8 @@ gfs_pio_internal_set_view_section(GFS_File gf, char *host)
 	gfarm_timerval_t t1, t2;
 	int host_assigned = 0;
 	gfarm_int32_t port;
-	int retry = 0, sleep_interval = 5;
-	/* wait at least 10 min = maximum reconnection interval of gfsd */
-	int max_retry = 7;
+	int sleep_interval = 1, sleep_max_interval = 512;
+	struct timeval expiration_time;
 
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
@@ -565,17 +507,20 @@ gfs_pio_internal_set_view_section(GFS_File gf, char *host)
 	}
 	gf->view_context = vc;
 
+	gettimeofday(&expiration_time, NULL);
+	expiration_time.tv_sec += gfarm_no_file_system_node_timeout;
 	for (;;) {
 		if (host == NULL) {
 			e = gfarm_schedule_file(gf, &host, &port);
 			/* reschedule another host */
 			if (e == GFARM_ERRMSG_NO_FILESYSTEM_NODE &&
-			    ++retry <= max_retry) {
+			    !gfarm_timeval_is_expired(&expiration_time)) {
 				gflog_warning(GFARM_MSG_UNFIXED,
 				    "sleep %d sec: %s", sleep_interval,
 				    gfarm_error_string(e));
 				sleep(sleep_interval);
-				sleep_interval *= 2;
+				if (sleep_interval < sleep_max_interval)
+					sleep_interval *= 2;
 				continue;
 			}
 			if (e == GFARM_ERR_NO_ERROR)
@@ -592,13 +537,14 @@ gfs_pio_internal_set_view_section(GFS_File gf, char *host)
 			 */
 			if ((e == GFARM_ERRMSG_NO_FILESYSTEM_NODE ||
 			    gfs_client_is_connection_error(e)) &&
-			    ++retry <= max_retry) {
+			    !gfarm_timeval_is_expired(&expiration_time)) {
 				gflog_warning(GFARM_MSG_UNFIXED,
 				    "sleep %d sec: %s", sleep_interval,
 				    gfarm_error_string(e));
 				host = NULL;
 				sleep(sleep_interval);
-				sleep_interval *= 2;
+				if (sleep_interval < sleep_max_interval)
+					sleep_interval *= 2;
 				continue;
 			}
 		}
