@@ -1,3 +1,4 @@
+#include <pthread.h>	/* db_access.h currently needs this */
 #include <assert.h>
 #include <stdarg.h> /* for "gfp_xdr.h" */
 #include <stdlib.h>
@@ -66,7 +67,7 @@ gfm_server_compound_on_error(struct peer *peer, int from_client, int skip,
 		return (GFARM_ERR_NO_ERROR);
 	if (level < 1) /* there isn't COMPOUND_BEGIN ... END block around */
 		e = GFARM_ERR_INVALID_ARGUMENT;
-	else 
+	else
 		*on_errorp = on_error;
 	return (gfm_server_put_reply(peer, "compound_on_error",
 	    GFARM_ERR_NO_ERROR, ""));
@@ -94,7 +95,7 @@ gfm_server_get_fd(struct peer *peer, int from_client, int skip)
 	else
 		e = peer_fdpair_externalize_current(peer);
 	giant_unlock();
-	
+
 	return (gfm_server_put_reply(peer, "get_fd", e, "i", fd));
 }
 
@@ -121,7 +122,7 @@ gfm_server_put_fd(struct peer *peer, int from_client, int skip)
 		e = peer_fdpair_externalize_current(peer);
 	}
 	giant_unlock();
-	
+
 	return (gfm_server_put_reply(peer, "put_fd", e, ""));
 }
 
@@ -143,7 +144,7 @@ gfm_server_save_fd(struct peer *peer, int from_client, int skip)
 	else
 		e = peer_fdpair_save(peer);
 	giant_unlock();
-	
+
 	return (gfm_server_put_reply(peer, "save_fd", e, ""));
 }
 
@@ -165,7 +166,7 @@ gfm_server_restore_fd(struct peer *peer, int from_client, int skip)
 	else
 		e = peer_fdpair_restore(peer);
 	giant_unlock();
-	
+
 	return (gfm_server_put_reply(peer, "restore_fd", e, ""));
 }
 
@@ -561,8 +562,8 @@ gfm_server_fstat(struct peer *peer, int from_client, int skip)
 	    st.st_ino, st.st_gen, st.st_mode, st.st_nlink,
 	    st.st_user, st.st_group, st.st_size,
 	    st.st_ncopy,
-	    st.st_atimespec.tv_sec, st.st_atimespec.tv_nsec, 
-	    st.st_mtimespec.tv_sec, st.st_mtimespec.tv_nsec, 
+	    st.st_atimespec.tv_sec, st.st_atimespec.tv_nsec,
+	    st.st_mtimespec.tv_sec, st.st_mtimespec.tv_nsec,
 	    st.st_ctimespec.tv_sec, st.st_ctimespec.tv_nsec);
 	if (e == GFARM_ERR_NO_ERROR) {
 		free(st.st_user);
@@ -1031,7 +1032,7 @@ gfm_server_flink(struct peer *peer, int from_client, int skip)
 	else if ((e = db_begin(msg)) != GFARM_ERR_NO_ERROR)
 		;
 	else {
-                e = inode_create_link(base, name, process, src);
+		e = inode_create_link(base, name, process, src);
 		db_end(msg);
 	}
 
@@ -1328,7 +1329,7 @@ gfm_server_getdirents(struct peer *peer, int from_client, int skip)
 				inode_accessed(inode);
 		}
 	}
-	
+
 	giant_unlock();
 	e2 = gfm_server_put_reply(peer, "getdirents", e, "i", n);
 	if (e2 == GFARM_ERR_NO_ERROR && e == GFARM_ERR_NO_ERROR) {
@@ -1404,7 +1405,7 @@ gfm_server_getdirentsplus(struct peer *peer, int from_client, int skip)
 				inode_accessed(inode);
 		}
 	}
-	
+
 	giant_unlock();
 	e2 = gfm_server_put_reply(peer, "getdirentsplus", e, "i", n);
 	if (e2 == GFARM_ERR_NO_ERROR && e == GFARM_ERR_NO_ERROR) {
@@ -1416,8 +1417,8 @@ gfm_server_getdirentsplus(struct peer *peer, int from_client, int skip)
 			    st->st_ino, st->st_gen, st->st_mode, st->st_nlink,
 			    st->st_user, st->st_group, st->st_size,
 			    st->st_ncopy,
-			    st->st_atimespec.tv_sec, st->st_atimespec.tv_nsec, 
-			    st->st_mtimespec.tv_sec, st->st_mtimespec.tv_nsec, 
+			    st->st_atimespec.tv_sec, st->st_atimespec.tv_nsec,
+			    st->st_mtimespec.tv_sec, st->st_mtimespec.tv_nsec,
 			    st->st_ctimespec.tv_sec, st->st_ctimespec.tv_nsec);
 			if (e2 != GFARM_ERR_NO_ERROR) {
 				gflog_warning(GFARM_MSG_1000387,
@@ -1491,7 +1492,7 @@ gfm_server_seek(struct peer *peer, int from_client, int skip)
 			    offset);
 		}
 	}
-	
+
 	giant_unlock();
 	return (gfm_server_put_reply(peer, "seek", e, "l", offset));
 }
@@ -1824,7 +1825,7 @@ gfm_server_replica_list_by_name(struct peer *peer, int from_client, int skip)
 	else if ((e = process_get_file_inode(process, fd, &inode))
 	    != GFARM_ERR_NO_ERROR)
 		;
-	else 
+	else
 		e = inode_replica_list_by_name(inode, &n, &hosts);
 
 	giant_unlock();
