@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 4563 2010-03-25 12:31:16Z tatebe $
+ * $Id: gfmd.c 4751 2010-06-16 13:53:28Z tatebe $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -96,7 +96,7 @@ gfm_server_protocol_extension_default(struct peer *peer,
 
 /* this interface is made as a hook for a private extension */
 gfarm_error_t (*gfm_server_protocol_extension)(struct peer *,
-	int, int, int, gfarm_int32_t, gfarm_int32_t *, gfarm_error_t *) = 
+	int, int, int, gfarm_int32_t, gfarm_int32_t *, gfarm_error_t *) =
 		gfm_server_protocol_extension_default;
 
 gfarm_error_t
@@ -942,8 +942,6 @@ void (*gfmd_modules_init)(int); /* intentionally remains uninitialized */
 int
 main(int argc, char **argv)
 {
-	extern char *optarg;
-	extern int optind;
 	gfarm_error_t e;
 	char *config_file = NULL, *port_number = NULL;
 	int syslog_level = -1;
@@ -1101,6 +1099,9 @@ main(int argc, char **argv)
 		    "create_detached_thread(db_thread): %s",
 			    gfarm_error_string(e));
 
+	/* check and repair nlink */
+	inode_nlink_check();
+
 	accepting_loop(sock);
 
 	/*NOTREACHED*/
