@@ -1,5 +1,5 @@
 /*
- * $Id: inode.c 4600 2010-03-29 15:53:10Z tatebe $
+ * $Id: inode.c 4771 2010-06-23 20:50:24Z tatebe $
  */
 
 #include <assert.h>
@@ -59,6 +59,7 @@ struct inode {
 	gfarm_ino_t i_number;
 	gfarm_uint64_t i_gen;
 	gfarm_uint64_t i_nlink;
+	gfarm_uint64_t i_nlink_ini;
 	gfarm_off_t i_size;
 	struct user *i_user;
 	struct group *i_group;
@@ -217,6 +218,17 @@ inode_cksum_set(struct file_opening *fo,
 
 	assert(ios != NULL);
 
+	if (strlen(cksum_type) > GFM_PROTO_CKSUM_TYPE_MAXLEN) {
+		gflog_debug(GFARM_MSG_UNFIXED,
+		    "too long cksum type: \"%s\"", cksum_type);
+		return (GFARM_ERR_INVALID_ARGUMENT);
+	}
+	if (cksum_len > GFM_PROTO_CKSUM_MAXLEN) {
+		gflog_debug(GFARM_MSG_UNFIXED,
+		    "too long cksum (type: \"%s\"): %d bytes",
+		    cksum_type, (int)cksum_len);
+		return (GFARM_ERR_INVALID_ARGUMENT);
+	}
 	if (!inode_is_file(fo->inode))
 		return (GFARM_ERR_BAD_FILE_DESCRIPTOR);
 
@@ -305,7 +317,7 @@ inode_cksum_get(struct file_opening *fo,
 static void
 xattrs_init(struct xattrs *xattrs)
 {
-	xattrs->head = xattrs->tail =NULL;
+	xattrs->head = xattrs->tail = NULL;
 }
 
 static void
@@ -456,6 +468,7 @@ inode_alloc_num(gfarm_ino_t inum)
 		inode->u.l.prev->u.l.next = inode->u.l.next;
 		inode->i_gen++;
 	}
+	inode->i_nlink_ini = 0;
 	inode->u.c.state = NULL;
 	pthread_mutex_lock(&total_num_inodes_mutex);
 	++total_num_inodes;
@@ -479,7 +492,7 @@ static void
 inode_clear(struct inode *inode)
 {
 	inode->i_mode = INODE_MODE_FREE;
-	inode->i_nlink = 0;
+	inode->i_nlink = inode->i_nlink_ini = 0;
 	/* add to the inode_free_list */
 	inode->u.l.prev = &inode_free_list;
 	inode->u.l.next = inode_free_list.u.l.next;
@@ -676,6 +689,18 @@ inode_get_nlink(struct inode *inode)
 	return (inode->i_nlink);
 }
 
+static gfarm_int64_t
+inode_get_nlink_ini(struct inode *inode)
+{
+	return (inode->i_nlink_ini);
+}
+
+static void
+inode_increment_nlink_ini(struct inode *inode)
+{
+	++inode->i_nlink_ini;
+}
+
 struct user *
 inode_get_user(struct inode *inode)
 {
@@ -1015,8 +1040,11 @@ inode_lookup_basename(struct inode *parent, const char *name, int len,
 		return (GFARM_ERR_NO_ERROR);
 	} else if (memchr(name, '/', len) != NULL)
 		return (GFARM_ERR_INVALID_ARGUMENT);
-	if (len > GFS_MAXNAMLEN)
-		len = GFS_MAXNAMLEN;
+	if (len > GFS_MAXNAMLEN) {
+		gflog_debug(GFARM_MSG_UNFIXED,
+		    "op %d: too long file name: \"%s\"", op, name);
+		return (GFARM_ERR_FILE_NAME_TOO_LONG);
+	}
 	if (op != INODE_CREATE && op != INODE_CREATE_EXCLUSIVE &&
 	    op != INODE_LINK) {
 		entry = dir_lookup(parent->u.c.s.d.entries, name, len);
@@ -1328,6 +1356,13 @@ inode_create_symlink(struct inode *base, char *name,
 {
 	struct inode *inode;
 
+	if (strlen(source_path) > GFARM_PATH_MAX) {
+		gflog_debug(GFARM_MSG_UNFIXED,
+		    "create symlink \"%s\"  \"%s\": too long source path",
+		    source_path, name);
+		return (GFARM_ERR_FILE_NAME_TOO_LONG);
+	}
+
 	return (inode_lookup_relative(base, name, GFS_DT_LNK,
 	    INODE_CREATE_EXCLUSIVE, process_get_user(process),
 	    0777, source_path, &inode, NULL));
@@ -1348,7 +1383,7 @@ inode_create_link(struct inode *base, char *name,
 {
 	gfarm_error_t e;
 
-	e = inode_create_link_internal(base, name, process_get_user(process), 
+	e = inode_create_link_internal(base, name, process_get_user(process),
 	    inode);
 	if (e != GFARM_ERR_NO_ERROR)
 		return (e);
@@ -1668,6 +1703,11 @@ inode_open(struct file_opening *fo)
 	}
 	if ((accmode_to_op(fo->flag) & GFS_W_OK) != 0)
 		++ios->u.f.writers;
+	if ((fo->flag & GFARM_FILE_TRUNC) != 0) {
+		inode_status_changed(inode);
+		inode_modified(inode);
+		inode_set_size(inode, 0);
+	}
 
 	fo->opening_prev = &ios->openings;
 	fo->opening_next = ios->openings.opening_next;
@@ -1931,7 +1971,7 @@ inode_schedule_file_reply_default(struct inode *inode, struct peer *peer,
 	for (copy = inode->u.c.s.f.copies; copy != NULL;
 	    copy = copy->host_next) {
 		if (copy->valid && host_is_up(copy->host))
-		    n++;
+			n++;
 	}
 	e_save = host_schedule_reply_n(peer, n, diag);
 	for (copy = inode->u.c.s.f.copies; copy != NULL;
@@ -2181,6 +2221,7 @@ dir_entry_add_one(void *closure,
 		    "dir_entry_add_one: already exists ");
 	} else {
 		dir_entry_set_inode(entry, entry_inode);
+		inode_increment_nlink_ini(entry_inode);
 	}
 	free(entry_name);
 }
@@ -2220,7 +2261,7 @@ inode_init(void)
 	st.st_user = strdup(ADMIN_USER_NAME);
 	st.st_group = strdup(ADMIN_GROUP_NAME);
 	touch(&st.st_atimespec);
- 	st.st_ctimespec = st.st_mtimespec = st.st_atimespec;
+	st.st_ctimespec = st.st_mtimespec = st.st_atimespec;
 	/* inode_add_one will free(st). need to call db_inode_add before it */
 	e = db_inode_add(&st);
 	inode_add_one(NULL, &st);
@@ -2268,6 +2309,53 @@ dead_file_copy_init(void)
 		    "loading deadfilecopy: %s", gfarm_error_string(e));
 }
 
+static void
+nlink_check(void *closure, struct inode *inode)
+{
+	gfarm_error_t e;
+
+	/* XXX - nlink of a directory is the constant 2 for now */
+	if (inode_is_dir(inode)) {
+		if (inode_get_nlink(inode) != 2) {
+			gflog_warning(GFARM_MSG_UNFIXED,
+			    "directory inode %lld nlink %lld should be 2 "
+			    "(really %lld): fixed",
+			    (unsigned long long)inode_get_number(inode),
+			    (unsigned long long)inode_get_nlink(inode),
+			    (unsigned long long)inode_get_nlink_ini(inode));
+			inode->i_nlink = 2;
+		} else
+			return;
+	} else if (inode_get_nlink_ini(inode) == 0) {
+		gflog_warning(GFARM_MSG_UNFIXED,
+		    "inode %lld is not referenced",
+		    (unsigned long long)inode_get_number(inode));
+		/* XXX - move to the /lost+found directory */
+		return;
+	} else if (inode_get_nlink(inode) != inode_get_nlink_ini(inode)) {
+		gflog_warning(GFARM_MSG_UNFIXED,
+		    "inode %lld nlink %lld should be %lld: fixed",
+		    (unsigned long long)inode_get_number(inode),
+		    (unsigned long long)inode_get_nlink(inode),
+		    (unsigned long long)inode_get_nlink_ini(inode));
+		inode->i_nlink = inode_get_nlink_ini(inode);
+	} else
+		return;
+	e = db_inode_nlink_modify(
+	    inode_get_number(inode), inode_get_nlink(inode));
+	if (e != GFARM_ERR_NO_ERROR)
+		gflog_error(GFARM_MSG_UNFIXED,
+		    "db_inode_nlink_modify(%lld): %s",
+		    (unsigned long long)inode->i_number,
+		    gfarm_error_string(e));
+}
+
+void
+inode_nlink_check(void)
+{
+	inode_lookup_all(NULL, nlink_check);
+}
+
 void
 dir_entry_init(void)
 {
